from pyewjn.dielectric.nam_dielectric_coefficient_approximator import get_nam_dielectric
from pyewjn.dielectric.lindhard_dielectric import get_lindhard_dielectric

__all__ = ["get_nam_dielectric", "get_lindhard_dielectric"]
